@@ -11,14 +11,10 @@
 //! threads, and are the building blocks of other concurrent types.
 //!
 //! This library defines a generic atomic wrapper type `Atomic<T>` for all
-//! `T: NoUninit` types.
+//! `T: Copy` types.
 //! Atomic types present operations that, when used correctly, synchronize
 //! updates between threads.
 //!
-//! The `NoUninit` bound is from the [bytemuck] crate, and indicates that a
-//! type has no internal padding bytes. You will need to derive or implement
-//! this trait for all types used with `Atomic<T>`.
-//!
 //! Each method takes an `Ordering` which represents the strength of
 //! the memory barrier for that operation. These orderings are the
 //! same as [LLVM atomic orderings][1].
@@ -33,8 +29,6 @@
 //! Most atomic types may be stored in static variables, initialized using
 //! the `const fn` constructors. Atomic statics are often used for lazy global
 //! initialization.
-//!
-//! [bytemuck]: https://docs.rs/bytemuck
 
 #![warn(missing_docs)]
 #![warn(rust_2018_idioms)]
@@ -55,8 +49,6 @@ use core::fmt;
 #[cfg(feature = "std")]
 use std::panic::RefUnwindSafe;
 
-use bytemuck::NoUninit;
-
 #[cfg(feature = "fallback")]
 mod fallback;
 mod ops;
@@ -79,19 +71,19 @@ unsafe impl<T: Copy + Send> Sync for Atomic<T> {}
 // `Atomic` API does not allow doing any panic-inducing operation after writing
 // to the target object.
 #[cfg(feature = "std")]
-impl<T: RefUnwindSafe> RefUnwindSafe for Atomic<T> {}
+impl<T: Copy + RefUnwindSafe> RefUnwindSafe for Atomic<T> {}
 
-impl<T: Default> Default for Atomic<T> {
+impl<T: Copy + Default> Default for Atomic<T> {
     #[inline]
     fn default() -> Self {
         Self::new(Default::default())
     }
 }
 
-impl<T: NoUninit + fmt::Debug> fmt::Debug for Atomic<T> {
+impl<T: Copy + fmt::Debug> fmt::Debug for Atomic<T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_tuple("Atomic")
-            .field(&self.load(Ordering::Relaxed))
+            .field(&self.load(Ordering::SeqCst))
             .finish()
     }
 }
@@ -116,7 +108,7 @@ impl<T> Atomic<T> {
     }
 }
 
-impl<T: NoUninit> Atomic<T> {
+impl<T: Copy> Atomic<T> {
     #[inline]
     fn inner_ptr(&self) -> *mut T {
         self.v.get() as *mut T
@@ -397,46 +389,18 @@ macro_rules! atomic_ops_unsigned {
 atomic_ops_signed! { i8 i16 i32 i64 isize i128 }
 atomic_ops_unsigned! { u8 u16 u32 u64 usize u128 }
 
-#[cfg(feature = "serde")]
-mod serde_impl;
-
 #[cfg(test)]
 mod tests {
     use super::{Atomic, Ordering::*};
-    use bytemuck::NoUninit;
     use core::mem;
 
-    #[derive(Copy, Clone, Eq, PartialEq, Debug, Default, NoUninit)]
-    #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-    #[repr(C)]
+    #[derive(Copy, Clone, Eq, PartialEq, Debug, Default)]
     struct Foo(u8, u8);
-
-    #[derive(Copy, Clone, Eq, PartialEq, Debug, Default, NoUninit)]
-    #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-    #[repr(C)]
+    #[derive(Copy, Clone, Eq, PartialEq, Debug, Default)]
     struct Bar(u64, u64);
-
-    #[derive(Copy, Clone, Eq, PartialEq, Debug, Default, NoUninit)]
-    #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-    #[repr(C)]
+    #[derive(Copy, Clone, Eq, PartialEq, Debug, Default)]
     struct Quux(u32);
 
-    #[cfg(feature = "serde")]
-    fn assert_serde<T>(atomic: &Atomic<T>, value: T)
-    where
-        T: NoUninit
-            + PartialEq
-            + std::fmt::Debug
-            + for<'a> serde::Deserialize<'a>
-            + serde::Serialize,
-    {
-        let s = serde_json::to_string(atomic).unwrap();
-        assert_eq!(s, serde_json::to_string(&value).unwrap());
-
-        let x: Atomic<T> = serde_json::from_str(&s).unwrap();
-        assert_eq!(x.load(SeqCst), value);
-    }
-
     #[test]
     fn atomic_bool() {
         let a = Atomic::new(false);
@@ -454,9 +418,6 @@ mod tests {
         assert_eq!(a.fetch_or(true, SeqCst), false);
         assert_eq!(a.fetch_xor(false, SeqCst), true);
         assert_eq!(a.load(SeqCst), true);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, true);
     }
 
     #[test]
@@ -478,9 +439,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(-25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -504,9 +462,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(-25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -530,9 +485,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(-25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -556,9 +508,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(-25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -582,9 +531,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(-25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -604,9 +550,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(-25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -627,9 +570,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -653,9 +593,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -679,9 +616,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -705,9 +639,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -731,9 +662,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -753,9 +681,6 @@ mod tests {
         assert_eq!(a.fetch_min(30, SeqCst), 71);
         assert_eq!(a.fetch_max(25, SeqCst), 30);
         assert_eq!(a.load(SeqCst), 30);
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, 30);
     }
 
     #[test]
@@ -775,9 +700,6 @@ mod tests {
             Ok(Foo(2, 2))
         );
         assert_eq!(a.load(SeqCst), Foo(3, 3));
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, Foo(3, 3));
     }
 
     #[test]
@@ -797,9 +719,6 @@ mod tests {
             Ok(Bar(2, 2))
         );
         assert_eq!(a.load(SeqCst), Bar(3, 3));
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, Bar(3, 3));
     }
 
     #[test]
@@ -822,8 +741,5 @@ mod tests {
             Ok(Quux(2))
         );
         assert_eq!(a.load(SeqCst), Quux(3));
-
-        #[cfg(feature = "serde")]
-        assert_serde(&a, Quux(3));
     }
 }
