@@ -14,12 +14,19 @@ pub use super::syntax::*;
 pub(super) use super::paste;
 pub(super) use super::paste::{IdentFrag, IdentFragInfallible};
 
+pub(super) use general_context::{
+    DriverlessContext, GeneralContext, GeneralContextBuf, MissingContextError,
+};
+
 /// Context during expansion
 ///
 /// References the driver, and digested information about it.
 /// Also represents where in the driver we are,
 /// including repetition context.
-#[derive(Debug, Clone)]
+//
+// Has to be Copy because otherwise the borrow checker doesn't understand
+// what's going on with ctx_buf in Template::expand.
+#[derive(Debug, Clone, Copy)]
 pub struct Context<'c> {
     pub top: &'c syn::DeriveInput,
     pub template_crate: &'c syn::Path,
@@ -29,9 +36,18 @@ pub struct Context<'c> {
     pub field: Option<&'c WithinField<'c>>,
     pub within_loop: WithinLoop,
     pub pvariants: &'c [PreprocessedVariant<'c>],
-    pub definitions: Definitions<'c>,
+    pub defs: DefinitionsContext<'c>,
+}
+
+/// Part of the context relating to user-defined expansions and conditions
+#[derive(Debug, Clone, Copy, Default)]
+pub struct DefinitionsContext<'c> {
+    pub defs: Definitions<'c>,
     pub nesting_depth: u16,
-    pub nesting_parent: Option<(&'c Context<'c>, &'c DefinitionName)>,
+    pub nesting_parent: Option<(
+        &'c DefinitionsContext<'c>, //
+        &'c DefinitionName,
+    )>,
 }
 
 #[derive(Debug)]
@@ -87,7 +103,7 @@ pub struct Definitions<'c> {
 }
 
 /// Special processing instructions returned by
-/// [`special_before_element_hook`](SubstParseContext::special_before_element_hook)
+/// [`before_element_hook`](SpecialParseContext::before_element_hook)
 pub enum SpecialInstructions {
     /// This template is finished
     ///
@@ -172,30 +188,19 @@ pub trait SubstParseContext: Sized {
     /// Used by the template-scanning code, to report whether an `Xmeta`
     /// in the template justifies a value-bearing `Xmeta` attribute
     /// on/in the driver, or just a boolean.
-    fn meta_recog_usage() -> meta::Usage;
+    fn meta_recog_usage(m: &meta::SubstMeta<Self>) -> meta::Usage {
+        if m.default.is_some() {
+            meta::Usage::VALUE_ONLY
+        } else {
+            meta::Usage::VALUE
+        }
+    }
 
     /// For communicating through `parse_special`
     ///
     /// Used when parsing `${paste ..}` to handle
     /// the special `>`-based end condition.
-    type SpecialParseContext: Default;
-
-    /// Handle any special syntax for a special kind of template context.
-    ///
-    /// This method is called only when parsing multi-element [`Template`]s,
-    /// It's a hook, called before parsing each `TemplateElement`.
-    ///
-    /// It should consume any special syntax as appropriate,
-    ///
-    /// The default implementation is a no-op.
-    /// The only non-default implementation is in `paste.rs`, for `$<...>` -
-    /// see [`paste::AngleBrackets`].
-    fn special_before_element_hook(
-        _special: &mut Self::SpecialParseContext,
-        _input: ParseStream,
-    ) -> syn::Result<Option<SpecialInstructions>> {
-        Ok(None)
-    }
+    type SpecialParseContext: SpecialParseContext + Default;
 
     /// Parse using `f`, within parens in boolean context, not otherwise
     ///
@@ -236,6 +241,32 @@ pub trait SubstParseContext: Sized {
     }
 }
 
+pub trait SpecialParseContext {
+    /// Handle any special syntax for a special kind of template context.
+    ///
+    /// This method is called only when parsing multi-element [`Template`]s,
+    /// It's a hook, called before parsing each `TemplateElement`.
+    ///
+    /// It should consume any special syntax as appropriate,
+    ///
+    /// The default implementation is a no-op.
+    /// The only non-default implementation is in `paste.rs`, for `$<...>` -
+    /// see [`paste::AngleBrackets`].
+    fn before_element_hook(
+        &mut self,
+        _input: ParseStream,
+    ) -> syn::Result<Option<SpecialInstructions>>;
+}
+
+impl SpecialParseContext for () {
+    fn before_element_hook(
+        &mut self,
+        _input: ParseStream,
+    ) -> syn::Result<Option<SpecialInstructions>> {
+        Ok(None)
+    }
+}
+
 /// Inhabited iff this lexical content may expand to arbitrary tokens
 ///
 /// Calculated automatically
@@ -431,10 +462,10 @@ pub trait ExpansionOutput: SubstParseContext {
     ///
     /// Expansion errors are to be reported, and subsumed into `self`.
     /// Failures to write may be thrown.
-    fn dbg_expand(
+    fn dbg_expand<'c>(
         &mut self,
         kw_span: Span,
-        ctx: &Context,
+        ctx: GeneralContext<'c>,
         msg: &mut String,
         content: &Self::DbgContent,
     ) -> fmt::Result;
@@ -451,7 +482,11 @@ pub trait ExpansionOutput: SubstParseContext {
 /// It also avoids having to make all of these expansion methods
 /// members of the `ExpansionOutput` trait.
 pub trait Expand<O> {
-    fn expand(&self, ctx: &Context, out: &mut O) -> syn::Result<()>;
+    fn expand<'c>(
+        &self,
+        ctx: GeneralContext<'c>,
+        out: &mut O,
+    ) -> syn::Result<()>;
 }
 /// Convenience trait providing `fn expand(self)`, infallible version
 ///
@@ -459,7 +494,7 @@ pub trait Expand<O> {
 /// within the output accumulator
 /// and therefore do not need to return them.
 pub trait ExpandInfallible<O> {
-    fn expand(&self, ctx: &Context, out: &mut O);
+    fn expand<'c>(&self, ctx: GeneralContext<'c>, out: &mut O);
 }
 
 /// Accumulates tokens, or errors
@@ -543,9 +578,6 @@ impl SubstParseContext for TokenAccumulator {
     fn not_in_bool(_: &impl Spanned) -> syn::Result<()> {
         Ok(())
     }
-    fn meta_recog_usage() -> meta::Usage {
-        meta::Usage::Value
-    }
 
     type BoolOnly = Void;
     type ConcatOnly = Void;
@@ -633,10 +665,10 @@ impl ExpansionOutput for TokenAccumulator {
         self.0.map(|_: TokenStream| ())
     }
 
-    fn dbg_expand(
+    fn dbg_expand<'c>(
         &mut self,
         _kw_span: Span,
-        ctx: &Context,
+        ctx: GeneralContext<'c>,
         msg: &mut String,
         content: &Template<TokenAccumulator>,
     ) -> fmt::Result {
