@@ -172,6 +172,11 @@ impl<O: SubstParseContext> AnalyseRepeat for Subst<O> {
             SD::fdefine(..) => Some(RO::Fields),
             SD::vdefbody(..) => Some(RO::Variants),
             SD::paste(body, ..) => recurse!(body),
+            SD::paste_spanned(span, content, ..) => {
+                span.analyse_repeat(visitor)?;
+                content.analyse_repeat(visitor)?;
+                None
+            }
             SD::ChangeCase(body, ..) => recurse!(body),
             SD::concat(body, ..) => recurse!(body),
             SD::when(..) => None, // out-of-place when, ignore it
@@ -197,6 +202,7 @@ impl<O: SubstParseContext> AnalyseRepeat for Subst<O> {
             SD::For(..) => None,
             SD::False(..) | SD::True(..) => None, // condition: ignore.
             SD::error(..) => None,
+            SD::require_beta(..) => None,
             SD::ignore(content, _) => recurse!(content),
             SD::dbg(ddr) => recurse!(ddr.content_parsed),
             SD::dbg_all_keywords(..) => None,
@@ -324,7 +330,7 @@ impl<'w> WithinRepeatLevel<'w> for WithinField<'w> {
 
 impl<'c> Context<'c> {
     /// Returns an [`ErrorLoc`] for the current part of the driver
-    pub fn error_loc(&self) -> (Span, &'static str) {
+    pub fn error_loc(&self) -> ErrorLoc<'static> {
         if let Some(field) = &self.field {
             (field.field.span(), "in this field")
         } else if let Some(variant) =
@@ -392,34 +398,6 @@ impl<'c> Context<'c> {
         })?;
         Ok(r)
     }
-
-    pub fn display_for_dbg(&self) -> impl Display + '_ {
-        struct Adapter<'a>(&'a Context<'a>);
-
-        impl Display for Adapter<'_> {
-            fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-                let ctx = self.0;
-                write!(f, "for {}", &ctx.top.ident)?;
-                if let Some(wv) = &ctx.variant {
-                    if let Some(v) = &wv.variant {
-                        write!(f, "::{}", &v.ident)?;
-                    }
-                }
-                if let Some(wf) = &ctx.field {
-                    // we're just using the Display impl, any span will do
-                    let span = Span::call_site();
-                    write!(f, ".{}", &wf.fname(span))?;
-                }
-                if let Some(templ) = &ctx.template_name {
-                    let templ = templ.to_token_stream().to_string();
-                    write!(f, " from {}", templ)?;
-                }
-                Ok::<_, fmt::Error>(())
-            }
-        }
-
-        Adapter(self)
-    }
 }
 
 //---------- Fname ----------
@@ -478,29 +456,34 @@ impl<'w> WithinVariant<'w> {
 
 //---------- definitions (user-defined keywords) ----------
 
-impl<'c> Context<'c> {
+impl<'c> GeneralContext<'c> {
     pub fn find_definition<B>(
         &'c self,
         call: &'c DefinitionName,
-    ) -> syn::Result<Option<(&'c Definition<B>, Context<'c>)>>
+    ) -> syn::Result<Option<(&'c Definition<B>, GeneralContextBuf<'c>)>>
     where
         Definitions<'c>: AsRef<[&'c Definition<B>]>,
         B: 'static,
     {
-        let def = match self.definitions.find_raw(call) {
+        let def = match self.defs().defs.find_raw(call) {
             Some(y) => y,
             None => return Ok(None),
         };
 
-        let ctx = self.deeper(&def.name, call)?;
-        Ok(Some((def, ctx)))
+        let defs = self.defs().find_definition_deeper(&def.name, call)?;
+        let mut self_ = self.clone_buf();
+        *self_.as_mut() = defs;
+        Ok(Some((def, self_)))
     }
+}
 
-    fn deeper(
+impl<'c> DefinitionsContext<'c> {
+    /// Internal function for use by `find_definition`
+    pub fn find_definition_deeper(
         &'c self,
         def: &'c DefinitionName,
         call: &'c DefinitionName,
-    ) -> syn::Result<Context<'c>> {
+    ) -> syn::Result<DefinitionsContext<'c>> {
         let nesting_depth = self.nesting_depth + 1;
         let stack_entry = (self, call);
         if nesting_depth > NESTING_LIMIT {
@@ -546,7 +529,7 @@ impl<'c> Context<'c> {
             }
             return Err(errs);
         }
-        Ok(Context {
+        Ok(DefinitionsContext {
             nesting_depth,
             nesting_parent: Some(stack_entry),
             ..*self
