@@ -10,7 +10,8 @@ use super::framework::*;
 pub use SubstDetails as SD;
 pub use TemplateElement as TE;
 
-#[derive(Debug)]
+#[derive(Debug, Default)]
+// We deliberately don't make this Clone, to avoid cloning whole parse trees.
 pub struct Template<O: SubstParseContext> {
     /// `TD::Subst` is known not to contain `SD::when`
     pub elements: Vec<TemplateElement<O>>,
@@ -18,8 +19,9 @@ pub struct Template<O: SubstParseContext> {
 
 #[derive(Debug)]
 pub struct TopTemplate {
-    pub content: Template<TokenAccumulator>,
+    content: Template<TokenAccumulator>,
 }
+impl_deref!(TopTemplate, content, Template<TokenAccumulator>);
 
 /// `ARG` as per
 /// "Named and positional template arguments"
@@ -60,6 +62,7 @@ pub enum TemplateElement<O: SubstParseContext> {
 
 #[derive(Debug)]
 pub struct RepeatedTemplate<O: SubstParseContext> {
+    pub span: Span,
     pub template: Template<O>,
     #[allow(clippy::vec_box)]
     pub whens: Vec<Box<Subst<BooleanContext>>>,
@@ -138,6 +141,12 @@ pub enum SubstDetails<O: SubstParseContext> {
 
     // expansion manipulation
     paste(Template<paste::Items>, O::NotInBool),
+    paste_spanned(
+        Argument,
+        Argument<paste::Items>,
+        O::NotInBool,
+        beta::Enabled,
+    ),
     ChangeCase(
         Template<paste::Items>,
         paste::ChangeCase,
@@ -179,6 +188,7 @@ pub enum SubstDetails<O: SubstParseContext> {
 
     ignore(Template<O>, O::NotInBool),
     error(ExplicitError, O::NotInBool),
+    require_beta(beta::Enabled, O::NotInBool),
     dbg(DbgDumpRequest<O>),
     dbg_all_keywords(O::NotInBool),
 
@@ -194,6 +204,8 @@ pub struct Definition<B> {
 
 #[derive(Debug, Eq, PartialEq, Ord, PartialOrd)]
 pub struct DefinitionName(syn::Ident);
+impl_to_tokens!(DefinitionName, 0);
+impl_display!(DefinitionName, 0);
 
 #[derive(Debug)]
 pub enum DefinitionBody {
@@ -288,12 +300,6 @@ pub struct DbgDumpRequest<O: SubstParseContext> {
 /// Error returned by `DefinitionName::try_from(syn::Ident)`
 pub struct InvalidDefinitionName;
 
-impl ToTokens for DefinitionName {
-    fn to_tokens(&self, out: &mut TokenStream) {
-        self.0.to_tokens(out)
-    }
-}
-
 impl TryFrom<syn::Ident> for DefinitionName {
     type Error = InvalidDefinitionName;
     fn try_from(ident: syn::Ident) -> Result<Self, InvalidDefinitionName> {
@@ -315,12 +321,6 @@ impl TryFrom<syn::Ident> for DefinitionName {
     }
 }
 
-impl Display for DefinitionName {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        Display::fmt(&self.0, f)
-    }
-}
-
 impl Parse for DefinitionName {
     fn parse(input: ParseStream) -> syn::Result<Self> {
         let ident = input.call(Ident::parse_any)?;
@@ -427,6 +427,10 @@ impl TopTemplate {
         input: ParseStream,
         beta_enabled: Option<beta::Enabled>,
     ) -> syn::Result<Self> {
+        let beta_enabled = beta_enabled.ok_or(error_generator!(
+ "beta derive-deftly feature used, without `beta_deftly` template option"
+        ));
+
         let content = beta::with_maybe_enabled(
             //
             beta_enabled,
@@ -436,32 +440,25 @@ impl TopTemplate {
     }
 }
 
-impl Deref for TopTemplate {
-    type Target = Template<TokenAccumulator>;
-    fn deref(&self) -> &Template<TokenAccumulator> {
-        &self.content
-    }
-}
-
 impl<O: SubstParseContext> Parse for Template<O> {
     fn parse(input: ParseStream) -> syn::Result<Self> {
-        Template::parse_special(input, &mut Default::default())
+        Template::parse_special(input, &mut O::SpecialParseContext::default())
     }
 }
 
 impl<O: SubstParseContext> Template<O> {
-    fn parse_special(
+    pub fn parse_special<S: SpecialParseContext>(
         input: ParseStream,
-        special: &mut O::SpecialParseContext,
+        special: &mut S,
     ) -> syn::Result<Self> {
         TemplateWithWhens::parse_special(input, special)?.try_into()
     }
 }
 
 impl<O: SubstParseContext> TemplateWithWhens<O> {
-    fn parse_special(
+    fn parse_special<S: SpecialParseContext>(
         input: ParseStream,
-        mut special: &mut O::SpecialParseContext,
+        mut special: &mut S,
     ) -> syn::Result<Self> {
         // eprintln!("@@@@@@@@@@ PARSE {}", &input);
         let mut good = vec![];
@@ -469,7 +466,7 @@ impl<O: SubstParseContext> TemplateWithWhens<O> {
         while !input.is_empty() {
             let special = &mut special;
             match errors.handle_in(|| {
-                O::special_before_element_hook(special, input)
+                special.before_element_hook(input)
             }) {
                 Some(None) => {},
                 None | // error! quit parsing
@@ -521,12 +518,20 @@ impl<O: SubstParseContext> Deref for Argument<O> {
 ///
 /// Do not construct this yourself.
 /// Only `deescape_orig_dollar` is allowed to do that.
-pub enum OrigDollarDeescaped {
+#[derive(Debug, strum::EnumDiscriminants)]
+pub enum OrigDollarHandled {
     /// We didn't find any `orig_dollar`
     NotFound,
     /// We *did* find `orig_dollar`; if we now see *another* `$`
     /// things get even stranger.
-    Found,
+    Found(Span),
+}
+
+impl OrigDollarHandled {
+    // TODO MSRV 1.66, use strum 0.27 which has IntoDiscriminant trait
+    pub fn discriminant(&self) -> OrigDollarHandledDiscriminants {
+        self.into()
+    }
 }
 
 /// Skip over any `orig_dollar`
@@ -536,14 +541,14 @@ pub enum OrigDollarDeescaped {
 /// [`definition::escape_dollars`](escape_dollars).
 pub fn deescape_orig_dollar(
     input: ParseStream,
-) -> syn::Result<OrigDollarDeescaped> {
+) -> syn::Result<OrigDollarHandled> {
     input.step(|cursor| {
         let (found, rest) = (|| {
             let (ident, rest) = cursor.ident()?;
             (ident == "orig_dollar").then(|| ())?;
-            Some((OrigDollarDeescaped::Found, rest))
+            Some((OrigDollarHandled::Found(ident.span()), rest))
         })()
-        .unwrap_or((OrigDollarDeescaped::NotFound, *cursor));
+        .unwrap_or((OrigDollarHandled::NotFound, *cursor));
         Ok((found, rest))
     })
 }
@@ -594,11 +599,11 @@ impl<O: SubstParseContext> Parse for TemplateElement<O> {
                     // $$
                     let dollar: Punct = input.parse()?;
                     match deescaped {
-                        OrigDollarDeescaped::NotFound => {},
-                        OrigDollarDeescaped::Found => {
+                        OrigDollarHandled::NotFound => {},
+                        OrigDollarHandled::Found(_) => {
                             match deescape_orig_dollar(input)? {
-                                OrigDollarDeescaped::Found => {},
-                                OrigDollarDeescaped::NotFound => {
+                                OrigDollarHandled::Found(_) => {},
+                                OrigDollarHandled::NotFound => {
                                     return Err(dollar.error(
  "found `$orig_dollar $` not followed by another orig_dollar!"
                                     ))
@@ -764,8 +769,7 @@ impl ParseUsingSubkeywords for SubstVPat {
 
 impl<O: SubstParseContext> Subst<O> {
     /// Parses everything including a `$` (which we insist on)
-    #[allow(dead_code)] // This was once used for ${paste }
-    fn parse_entire(input: ParseStream) -> syn::Result<Self> {
+    pub fn parse_entire(input: ParseStream) -> syn::Result<Self> {
         let _dollar: Token![$] = input.parse()?;
         let deescaped = deescape_orig_dollar(input)?;
         let la = input.lookahead1();
@@ -779,7 +783,7 @@ impl<O: SubstParseContext> Subst<O> {
     fn parse_after_dollar(
         la: Lookahead1,
         input: ParseStream,
-        _deescaped: OrigDollarDeescaped,
+        _deescaped: OrigDollarHandled,
     ) -> syn::Result<Self> {
         if la.peek(token::Brace) {
             let exp;
@@ -970,6 +974,12 @@ impl<O: SubstParseContext> Parse for Subst<O> {
         }) }
 
         keyword! { paste(Template::parse(input)?, not_in_bool()?) }
+        keyword! { paste_spanned(
+            input.parse()?,
+            input.parse()?,
+            not_in_bool()?,
+            beta()?,
+        ) }
 
         keyword! { concat(
             Template::parse(input)?,
@@ -988,6 +998,7 @@ impl<O: SubstParseContext> Parse for Subst<O> {
         keyword! { select1(parse_if(input)?, not_in_bool()?) }
         keyword! { ignore(input.parse()?, not_in_bool()?) }
         keyword! { error(input.parse()?, not_in_bool()?) }
+        keyword! { require_beta(beta()?, not_in_bool()?) }
         keyword! { dbg(input.parse()?) }
         keyword! { dbg_all_keywords(not_in_bool()?) }
         keyword! { "crate": Crate(allow_tokens()?, not_in_bool()?) }
@@ -1052,14 +1063,17 @@ impl<O: SubstParseContext> Parse for DbgDumpRequest<O> {
 impl<O: SubstParseContext> DbgDumpRequest<O> {
     pub fn display_heading<'s>(
         &'s self,
-        ctx: &'s Context,
+        ctx: GeneralContext<'s>,
     ) -> impl Display + 's {
-        struct Adapter<'a, O: SubstParseContext>(
+        struct Adapter<'a, 'c, O: SubstParseContext>(
             &'a DbgDumpRequest<O>,
-            &'a Context<'a>,
+            GeneralContext<'c>,
         );
 
-        impl<O: SubstParseContext> Display for Adapter<'_, O> {
+        impl<O> Display for Adapter<'_, '_, O>
+        where
+            O: SubstParseContext,
+        {
             fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
                 let Adapter(ddr, ctx) = self;
                 if let Some(note) = &ddr.note {
@@ -1256,8 +1270,10 @@ impl<O: SubstParseContext> RepeatedTemplate<O> {
     ) -> Result<RepeatedTemplate<O>, syn::Error> {
         use TemplateWithWhens as TWW;
 
-        let TWW { elements } =
-            TWW::parse_special(input, &mut Default::default())?;
+        let TWW { elements } = TWW::parse_special::<O::SpecialParseContext>(
+            input,
+            &mut Default::default(),
+        )?;
 
         // split `when` off
         let mut elements = VecDeque::from(elements);
@@ -1292,6 +1308,7 @@ impl<O: SubstParseContext> RepeatedTemplate<O> {
 
         match over {
             Ok(over) => Ok(RepeatedTemplate {
+                span: span.open(),
                 over,
                 template,
                 whens,
