@@ -95,9 +95,10 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
                                 }
                             },
                             Some(FlatArg::Register { reg: Register::Dynamic(_, ref expr), .. }) => {
+                                let expr = delimited(expr);
                                 dynamics.push((0, quote_spanned!{ span=>
                                     {
-                                        let _dyn_reg: u8 = #expr;
+                                        let _dyn_reg: u8 = #expr.into();
                                         if _dyn_reg == #code {
                                             ::dynasmrt::riscv::invalid_register(#code);
                                         }
@@ -123,15 +124,20 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
 
             FlatArg::Register { span, reg: Register::Dynamic(_, ref expr) } => match *command {
                 Command::R(offset) => {
+                    let expr = delimited(expr);
                     dynamics.push((offset, quote_spanned!{ span=>
-                        ((#expr & 0x1F) as u32)
+                        {
+                            let _dyn_reg: u8 = #expr.into();
+                            (_dyn_reg & 0x1F) as u32
+                        }
                     }));
                 },
                 Command::Reven(offset) => {
                     let invalid_reg_mask: u8 = if ctx.target.is_embedded() { 0xF0 } else { 0xE0 };
+                    let expr = delimited(expr);
                     dynamics.push((offset, quote_spanned!{ span=>
                         {
-                            let _dyn_reg: u8 = #expr;
+                            let _dyn_reg: u8 = #expr.into();
                             if _dyn_reg & 0x1 != 0x0 || (_dyn_reg & #invalid_reg_mask) != 0 {
                                 ::dynasmrt::riscv::invalid_register(_dyn_reg);
                             }
@@ -141,9 +147,10 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
                 },
                 Command::Rno0(offset) => {
                     let invalid_reg_mask: u8 = if ctx.target.is_embedded() { 0xF0 } else { 0xE0 };
+                    let expr = delimited(expr);
                     dynamics.push((offset, quote_spanned!{ span=>
                         {
-                            let _dyn_reg: u8 = #expr;
+                            let _dyn_reg: u8 = #expr.into();
                             if _dyn_reg == 0x0 || (_dyn_reg & #invalid_reg_mask) != 0 {
                                 ::dynasmrt::riscv::invalid_register(_dyn_reg);
                             }
@@ -153,9 +160,10 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
                 },
                 Command::Rno02(offset) => {
                     let invalid_reg_mask: u8 = if ctx.target.is_embedded() { 0xF0 } else { 0xE0 };
+                    let expr = delimited(expr);
                     dynamics.push((offset, quote_spanned!{ span=>
                         {
-                            let _dyn_reg: u8 = #expr;
+                            let _dyn_reg: u8 = #expr.into();
                             if _dyn_reg == 0x0 || _dyn_reg == 0x2 || (_dyn_reg & #invalid_reg_mask) != 0 {
                                 ::dynasmrt::riscv::invalid_register(_dyn_reg);
                             }
@@ -165,9 +173,10 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
                 },
                 Command::Rpop(offset) => {
                     let invalid_reg_mask: u8 = if ctx.target.is_embedded() { 0xF0 } else { 0xE0 };
+                    let expr = delimited(expr);
                     dynamics.push((offset, quote_spanned!{ span=>
                         {
-                            let _dyn_reg: u8 = #expr;
+                            let _dyn_reg: u8 = #expr.into();
                             if _dyn_reg & 0x18 != 0x8 || (_dyn_reg & #invalid_reg_mask) != 0 {
                                 ::dynasmrt::riscv::invalid_register(_dyn_reg);
                             }
@@ -177,9 +186,10 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
                 },
                 Command::Rpops(offset) => {
                     let invalid_reg_mask: u8 = if ctx.target.is_embedded() { 0xF0 } else { 0xE0 };
+                    let expr = delimited(expr);
                     dynamics.push((offset, quote_spanned!{ span=>
                         {
-                            let _dyn_reg: u8 = #expr;
+                            let _dyn_reg: u8 = #expr.into();
                             if (1u32 << (_dyn_reg & 0x1F)) & 0x00_FC_03_00 == 0 || (_dyn_reg & #invalid_reg_mask) != 0 {
                                 ::dynasmrt::riscv::invalid_register(_dyn_reg);
                             }
@@ -191,9 +201,10 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
                     Some(FlatArg::Register { reg: Register::Static(id2), .. } ) => {
                         let code: u8 = id2.code();
                         let invalid_reg_mask: u8 = if ctx.target.is_embedded() { 0xF0 } else { 0xE0 };
+                        let expr = delimited(expr);
                         dynamics.push((offset, quote_spanned!{ span=>
                             {
-                                let _dyn_reg: u8 = #expr;
+                                let _dyn_reg: u8 = #expr.into();
                                 if (_dyn_reg == #code) || ((1u32 << (_dyn_reg & 0x1F)) & 0x00_FC_03_00 == 0) || (_dyn_reg & #invalid_reg_mask) != 0 {
                                     ::dynasmrt::riscv::invalid_register(_dyn_reg);
                                 }
@@ -203,10 +214,12 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
                     },
                     Some(FlatArg::Register { reg: Register::Dynamic(_, ref expr2), .. }) => {
                         let invalid_reg_mask: u8 = if ctx.target.is_embedded() { 0xF0 } else { 0xE0 };
+                        let expr = delimited(expr);
+                        let expr2 = delimited(expr2);
                         dynamics.push((offset, quote_spanned!{ span=>
                             {
-                                let _dyn_reg: u8 = #expr;
-                                let _dyn_reg_prev: u8 = #expr2;
+                                let _dyn_reg: u8 = #expr.into();
+                                let _dyn_reg_prev: u8 = #expr2.into();
                                 if (_dyn_reg == _dyn_reg_prev) || ((1u32 << (_dyn_reg & 0x1F)) & 0x00_FC_03_00 == 0) || (_dyn_reg & #invalid_reg_mask) != 0 {
                                     ::dynasmrt::riscv::invalid_register(_dyn_reg);
                                 }
@@ -771,6 +784,7 @@ pub(super) fn compile_instruction(ctx: &mut Context, data: MatchData) -> Result<
                     };
 
                     let imm_expr = if negated {
+                        let value = delimited(value);
                         quote_spanned!{ span=>
                             let _dyn_imm: i32 = -#value;
                         }
@@ -917,7 +931,7 @@ struct ImmediateEncoder<'a> {
 }
 
 impl<'a> ImmediateEncoder<'a> {
-    pub fn new(dynamic_value: &syn::Expr) -> ImmediateEncoder {
+    pub fn new(dynamic_value: &'a syn::Expr) -> ImmediateEncoder<'a> {
         #![allow(unexpected_cfgs)]
         let static_value;
 
