@@ -35,12 +35,14 @@ pub mod iterators;
 #[cfg(not(feature = "std"))]
 use std::vec::Vec;
 
+use crate::better_to_rebuild;
 use crate::core_iterators::*;
+use crate::store::{left, parent, right};
 use crate::store::{Index, Position, Store};
 use crate::TryReserveError;
+use equivalent::Equivalent;
 use iterators::*;
 
-use std::borrow::Borrow;
 use std::cmp::{Eq, Ord};
 #[cfg(feature = "std")]
 use std::collections::hash_map::RandomState;
@@ -60,6 +62,7 @@ use std::mem::replace;
 ///
 /// # Example
 /// ```rust
+/// # #[cfg(feature = "std")] {
 /// use priority_queue::PriorityQueue;
 ///
 /// let mut pq = PriorityQueue::new();
@@ -77,6 +80,7 @@ use std::mem::replace;
 /// for (item, _) in pq.into_sorted_iter() {
 ///     println!("{}", item);
 /// }
+/// # }
 /// ```
 #[derive(Clone, Debug)]
 #[cfg(feature = "std")]
@@ -151,8 +155,10 @@ where
     H: BuildHasher,
 {
     /// Creates an empty `PriorityQueue` with the specified hasher
-    pub fn with_hasher(hash_builder: H) -> Self {
-        Self::with_capacity_and_hasher(0, hash_builder)
+    pub const fn with_hasher(hash_builder: H) -> Self {
+        Self {
+            store: Store::with_hasher(hash_builder),
+        }
     }
 
     /// Creates an empty `PriorityQueue` with the specified capacity and hasher
@@ -170,7 +176,7 @@ where
 impl<I, P, H> PriorityQueue<I, P, H> {
     /// Returns an iterator in arbitrary order over the
     /// `(item, priority)` elements in the queue
-    pub fn iter(&self) -> Iter<I, P> {
+    pub fn iter(&self) -> Iter<'_, I, P> {
         self.store.iter()
     }
 
@@ -182,7 +188,7 @@ impl<I, P, H> PriorityQueue<I, P, H> {
 
     /// Clears the `PriorityQueue`, returning an iterator over the removed elements in arbitrary order.
     /// If the iterator is dropped before being fully consumed, it drops the remaining elements in arbitrary order.
-    pub fn drain(&mut self) -> Drain<I, P> {
+    pub fn drain(&mut self) -> Drain<'_, I, P> {
         self.store.drain()
     }
 
@@ -287,7 +293,7 @@ where
     ///
     /// It would be rebuilt even if no priority value would have been modified,
     /// but the procedure will not move anything, but just compare the priorities.
-    pub fn iter_mut(&mut self) -> IterMut<I, P, H> {
+    pub fn iter_mut(&mut self) -> IterMut<'_, I, P, H> {
         IterMut::new(self)
     }
 
@@ -379,6 +385,7 @@ where
     ///
     /// # Example
     /// ```
+    /// # #[cfg(feature = "std")] {
     /// # use priority_queue::PriorityQueue;
     /// let mut pq = PriorityQueue::new();
     ///
@@ -392,8 +399,9 @@ where
     ///
     /// assert_eq!(pq.peek(), Some((&"Bananas", &15)));
     /// assert_eq!(pq.into_vec(), vec!["Bananas"]);
+    /// # }
     /// ```
-    pub fn extract_if<F>(&mut self, predicate: F) -> ExtractIf<I, P, F, H>
+    pub fn extract_if<F>(&mut self, predicate: F) -> ExtractIf<'_, I, P, F, H>
     where
         F: FnMut(&mut I, &mut P) -> bool,
     {
@@ -418,6 +426,7 @@ where
     ///
     /// # Example
     /// ```
+    /// # #[cfg(feature = "std")] {
     /// # use priority_queue::PriorityQueue;
     /// let mut pq = PriorityQueue::new();
     ///
@@ -430,6 +439,7 @@ where
     /// }), None);
     ///
     /// assert_eq!(pq.pop(), Some(("Apples", 5)));
+    /// # }
     /// ```
     pub fn pop_if<F>(&mut self, predicate: F) -> Option<(I, P)>
     where
@@ -454,6 +464,7 @@ where
     ///
     /// # Example
     /// ```
+    /// # #[cfg(feature = "std")] {
     /// # use priority_queue::PriorityQueue;
     /// let mut pq = PriorityQueue::new();
     /// assert_eq!(pq.push("Apples", 5), None);
@@ -462,6 +473,7 @@ where
     /// assert_eq!(pq.get_priority("Apples"), Some(&6));
     /// assert_eq!(pq.push("Apples", 4), Some(6));
     /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// # }
     /// ```
     ///
     /// Computes in **O(log(N))** time.
@@ -511,6 +523,7 @@ where
     ///
     /// # Example
     /// ```
+    /// # #[cfg(feature = "std")] {
     /// # use priority_queue::PriorityQueue;
     /// let mut pq = PriorityQueue::new();
     /// assert_eq!(pq.push_increase("Apples", 5), None);
@@ -521,6 +534,7 @@ where
     /// // priority is returned.
     /// assert_eq!(pq.push_increase("Apples", 4), Some(4));
     /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// # }
     /// ```
     ///
     /// Computes in **O(log(N))** time.
@@ -549,6 +563,7 @@ where
     ///
     /// # Example
     /// ```
+    /// # #[cfg(feature = "std")] {
     /// # use priority_queue::PriorityQueue;
     /// let mut pq = PriorityQueue::new();
     /// assert_eq!(pq.push_decrease("Apples", 5), None);
@@ -559,6 +574,7 @@ where
     /// // priority is returned.
     /// assert_eq!(pq.push_decrease("Apples", 6), Some(6));
     /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// # }
     /// ```
     ///
     /// Computes in **O(log(N))** time.
@@ -578,6 +594,7 @@ where
     ///
     /// # Example
     /// ```
+    /// # #[cfg(feature = "std")] {
     /// # use priority_queue::PriorityQueue;
     /// let mut pq = PriorityQueue::new();
     /// assert_eq!(pq.change_priority("Apples", 5), None);
@@ -586,14 +603,14 @@ where
     /// assert_eq!(pq.get_priority("Apples"), Some(&6));
     /// assert_eq!(pq.change_priority("Apples", 4), Some(6));
     /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// # }
     /// ```
     ///
     /// The item is found in **O(1)** thanks to the hash table.
     /// The operation is performed in **O(log(N))** time.
     pub fn change_priority<Q>(&mut self, item: &Q, new_priority: P) -> Option<P>
     where
-        I: Borrow<Q>,
-        Q: Eq + Hash + ?Sized,
+        Q: ?Sized + Equivalent<I> + Hash,
     {
         self.store
             .change_priority(item, new_priority)
@@ -614,8 +631,7 @@ where
     /// The operation is performed in **O(log(N))** time (worst case).
     pub fn change_priority_by<Q, F>(&mut self, item: &Q, priority_setter: F) -> bool
     where
-        I: Borrow<Q>,
-        Q: Eq + Hash + ?Sized,
+        Q: ?Sized + Equivalent<I> + Hash,
         F: FnOnce(&mut P),
     {
         self.store
@@ -629,8 +645,7 @@ where
     /// Get the priority of an item, or `None`, if the item is not in the queue
     pub fn get_priority<Q>(&self, item: &Q) -> Option<&P>
     where
-        I: Borrow<Q>,
-        Q: Eq + Hash + ?Sized,
+        Q: ?Sized + Equivalent<I> + Hash,
     {
         self.store.get_priority(item)
     }
@@ -640,8 +655,7 @@ where
     /// Returns `true` if `item` is in the queue, `false` if it is not.
     pub fn contains<Q>(&self, item: &Q) -> bool
     where
-        I: Borrow<Q>,
-        Q: Eq + Hash + ?Sized,
+        Q: ?Sized + Equivalent<I> + Hash,
     {
         self.store.contains(item)
     }
@@ -650,8 +664,7 @@ where
     /// or `None` if the item is not in the queue.
     pub fn get<Q>(&self, item: &Q) -> Option<(&I, &P)>
     where
-        I: Borrow<Q>,
-        Q: Eq + Hash + ?Sized,
+        Q: ?Sized + Equivalent<I> + Hash,
     {
         self.store.get(item)
     }
@@ -668,8 +681,7 @@ where
     /// [`change_priority_by`](PriorityQueue::change_priority_by).
     pub fn get_mut<Q>(&mut self, item: &Q) -> Option<(&mut I, &P)>
     where
-        I: Borrow<Q>,
-        Q: Eq + Hash + ?Sized,
+        Q: ?Sized + Equivalent<I> + Hash,
     {
         self.store.get_mut(item)
     }
@@ -705,8 +717,7 @@ where
     /// The operation is performed in **O(log(N))** time (worst case).
     pub fn remove<Q>(&mut self, item: &Q) -> Option<(I, P)>
     where
-        I: Borrow<Q>,
-        Q: Eq + Hash + ?Sized,
+        Q: ?Sized + Equivalent<I> + Hash,
     {
         self.store.remove(item).map(|(item, priority, pos)| {
             if pos.0 < self.len() {
@@ -730,13 +741,6 @@ where
     }
 }
 
-impl<I, P, H> PriorityQueue<I, P, H>
-where
-    P: Ord,
-    I: Hash + Eq,
-{
-}
-
 impl<I, P, H> PriorityQueue<I, P, H>
 where
     P: Ord,
@@ -772,7 +776,7 @@ where
             self.store.swap(i, largest);
 
             i = largest;
-            let mut largestp = unsafe { self.store.get_priority_from_position(i) };
+            largestp = unsafe { self.store.get_priority_from_position(i) };
             l = left(i);
             if l.0 < self.len() {
                 let childp = unsafe { self.store.get_priority_from_position(l) };
@@ -977,47 +981,11 @@ where
     }
 }
 
-/// Compute the index of the left child of an item from its index
-#[inline(always)]
-const fn left(i: Position) -> Position {
-    Position((i.0 * 2) + 1)
-}
-/// Compute the index of the right child of an item from its index
-#[inline(always)]
-const fn right(i: Position) -> Position {
-    Position((i.0 * 2) + 2)
-}
-/// Compute the index of the parent element in the heap from its index
-#[inline(always)]
-const fn parent(i: Position) -> Position {
-    Position((i.0 - 1) / 2)
-}
-
-#[inline(always)]
-const fn log2_fast(x: usize) -> usize {
-    (usize::BITS - x.leading_zeros() - 1) as usize
-}
-
-// `rebuild` takes O(len1 + len2) operations
-// and about 2 * (len1 + len2) comparisons in the worst case
-// while `extend` takes O(len2 * log_2(len1)) operations
-// and about 1 * len2 * log_2(len1) comparisons in the worst case,
-// assuming len1 >= len2.
-fn better_to_rebuild(len1: usize, len2: usize) -> bool {
-    // log(1) == 0, so the inequation always falsy
-    // log(0) is inapplicable and produces panic
-    if len1 <= 1 {
-        return false;
-    }
-
-    2 * (len1 + len2) < len2 * log2_fast(len1)
-}
-
 #[cfg(feature = "serde")]
 #[cfg_attr(docsrs, doc(cfg(feature = "serde")))]
 mod serde {
-    use std::cmp::{Eq, Ord};
-    use std::hash::{BuildHasher, Hash};
+    use core::cmp::{Eq, Ord};
+    use core::hash::{BuildHasher, Hash};
 
     use serde::de::{Deserialize, Deserializer};
     use serde::ser::{Serialize, Serializer};
