@@ -74,7 +74,10 @@ trait TestCase: Sized {
 }
 
 impl TestCase for Passwd<Option<Vec<u8>>> {
-    type Output<S> = Passwd<S> where S: Eq + Debug;
+    type Output<S>
+        = Passwd<S>
+    where
+        S: Eq + Debug;
 
     fn try_into_string(
         ent: Passwd<Box<[u8]>>,
@@ -140,7 +143,10 @@ impl TestCase for Passwd<Option<Vec<u8>>> {
 }
 
 impl TestCase for Group<Option<Vec<u8>>> {
-    type Output<S> = Group<S> where S: Eq + Debug;
+    type Output<S>
+        = Group<S>
+    where
+        S: Eq + Debug;
 
     fn try_into_string(
         ent: Self::Output<Box<[u8]>>,
@@ -385,7 +391,7 @@ where
             buffer.push(T::to_libc(ent, a.clone())?);
         }
         buffer.push(T::Output::zero());
-        let (p, _, _) = Vec::into_raw_parts(buffer);
+        let (p, _, _, _) = Vec::into_raw_parts_with_alloc(buffer);
         Ok(p)
     }
 }
