@@ -1,22 +1,21 @@
-use core::borrow::Borrow;
-
-#[cfg(any(feature = "alloc", feature = "std"))]
 use alloc::string::String;
+use core::borrow::Borrow;
 
 use crate::{TiSlice, TiVec};
 
-/// Helper trait for [`[T]::concat`](../../std/primitive.slice.html#method.concat).
+/// A helper trait for [`TiSlice::concat`](crate::TiSlice#method.concat).
 pub trait Concat<Item: ?Sized> {
     /// The resulting type after concatenation
     type Output;
 
-    /// Implementation of [`[T]::concat`](../../std/primitive.slice.html#method.concat)
+    /// Implementation of [`TiSlice::concat`](crate::TiSlice#method.concat)
     fn concat(slice: &Self) -> Self::Output;
 }
 
 impl<K, V: Borrow<str>> Concat<str> for TiSlice<K, V> {
     type Output = String;
 
+    #[inline]
     fn concat(slice: &Self) -> Self::Output {
         slice.raw.concat()
     }
@@ -25,6 +24,7 @@ impl<K, V: Borrow<str>> Concat<str> for TiSlice<K, V> {
 impl<K, T: Clone, V: Borrow<[T]>> Concat<T> for TiSlice<K, V> {
     type Output = TiVec<K, T>;
 
+    #[inline]
     fn concat(slice: &Self) -> Self::Output {
         slice.raw.concat().into()
     }
